#!/bin/bash

#define constant file types stored in current_file_type

declare -r CSVFILES="CSVFILES:"
declare -r OLDCSVFILES="OLDCSVFILES:"
declare -r GIFFILES="GIFFILES:"
declare -r XMLFILES="XMLFILES:"
declare -r LOADFILES="LOADFILES:"
declare -r FIRMWAREFILES="FIRMWARE:"
declare -r OLDFIRMWAREFILES="OLDFIRMWARE:"
declare -r HELPFILES="HELPFILES:"

#define destination directories for the different file types
declare -r CSVDESTDIR="/usr/local/cm/db/csv/products"
declare -r GIFUSERDESTDIR="$TOMCAT_HOME/webapps/ccmuser/images"
declare -r GIFADMINDESTDIR="$TOMCAT_HOME/webapps/ccmadmin/images"
declare -r GIFSERVICEDESTDIR="$TOMCAT_HOME/webapps/ccmservice/images"
declare -r XMLDESTDIR="/usr/local/cm/db/xml/xml"
declare -r FIRMWAREDESTDIR="/usr/local/cm/tftp"
declare -r LOADFILEDESTDIR="/usr/local/cm/db/loadinfo"

# define current and install log directories
declare -r CURDIR=`pwd`

# define installdb executable
declare -r INSTALLDB=/usr/local/cm/bin/installdb

#define constant return values used by get_config_input

declare -r FILE_TYPE_CHANGED=1
declare -r END_OF_INPUT=2

#initialize global variables

#current_file_type and current_input are shared between
# read_config_file and get_config_input
current_file_type=""
current_input=""

# initialize the global lists of files contained in this copfile
csv_files=""
old_csv_files=""
gif_files=""
xml_files=""
firmware_files=""
old_firmware_files=""
load_files=""
help_files=""

# get_config_input reads an current_input line from the config file and
#   - sets current_file_type and returns FILE_TYPE_CHANGED when the file
#     type changes
#   - returns END_OF_INPUT at end of file
#   - sets current_input and returns zero for the next file of the current type
#
# get_config_input expects stdin to be set to the config file by the caller

function get_config_input()
{
    # return 2 to indicate EOF
    read current_input || return $END_OF_INPUT

    # skip comments, including trailing comments
    current_input=${current_input%%#*}

    [ -z "$current_input" ] && return 0

    if echo $current_input | grep -q ':$' ; then
        current_file_type=$current_input
        return $FILE_TYPE_CHANGED
    fi

    # disallow '..' and '*' in file names
    if [ "${current_input}" != "${current_input%%..*}" ] ||
       [ "${current_input}" != "${current_input%%\**}" ] ; then
        echo "Bad file name ${current_input}: disallowed"
        current_input=""
        return 0
    fi

    return 0
}

# read_config_file sets global variables for all file types from the
# configuration file.  It calls get_config_input, setting global variables
# for the different file types as they are read in.
#
# arguments: configuration file

function read_config_file()
{
    # fix copstart.config if it was created on Windows
    sed -i 's/\x0D//g' ./copstart.config

    # redirect input from config file
    exec 0<./copstart.config
    while true; do

        get_config_input
        rc=$?

        #we're done if EOF
        [ $rc == $END_OF_INPUT ] && break

        #if the type of file changed, we get files of that type, now
        [ $rc == $FILE_TYPE_CHANGED ] && continue

        # skip empty input
        [ -z "$current_input" ] && continue

        # if we have a filename that doesn't exist and it's not an
        # old firmware file, complain and continue.
        # We need to handle several files on the same line.
        file_list=""
        for file in ${current_input} ; do
            
            if [ ! -f "${file}" ] &&
               [ "$current_file_type" != $OLDFIRMWAREFILES ] &&
               [ "$current_file_type" != $OLDCSVFILES ] ; then
                echo "$file does not exist"
                continue
            fi
            file_list="$file_list $file"
        done

        [ -z "$file_list" ] && continue

        current_input=$file_list

        case $current_file_type in

        $CSVFILES )
            csv_files="$csv_files $current_input"
            ;;
        $OLDCSVFILES )
            old_csv_files="$old_csv_files $current_input"
            ;;
        $GIFFILES )
            gif_files="$gif_files $current_input"
            ;;
        $XMLFILES )
            xml_files="$xml_files $current_input"
            ;;
        $FIRMWAREFILES )
            firmware_files="$firmware_files $current_input"
            ;;
        $OLDFIRMWAREFILES )
            old_firmware_files="$old_firmware_files $current_input"
            ;;
        $LOADFILES )
            load_files="$load_files $current_input"
            ;;
        $HELPFILES )
            help_files="$help_files $current_input"
            ;;
        *)
            ;;
        esac
    done

    # redirect stdin from /dev/null, for good measure
    exec 0</dev/null
}

read_config_file

echo "CSVFILES = $csv_files"
echo "OLDCSVFILES = $old_csv_files"
echo "GIFFILES = $gif_files"
echo "XMLFILES = $xml_files"
echo "FIRMWAREFILES = $firmware_files"
echo "OLDFIRMWAREFILES = $old_firmware_files"
echo "LOADFILES = $load_files"
echo "HELPFILES = $help_files"

# set dynamic library search for installdb
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/cm/lib
export LD_LIBRARY_PATH

# Context equals 'L2', 'options' or 'install'
if [ "${CONTEXT}" != "L2" ]
then 
declare -r L2_OPT=
else
declare -r L2_OPT=L2
fi

#
# change permissions/ownership/group and copy CSV files
#
if [ -n "$csv_files" ] ; then
    echo "chmod 644 $csv_files"
    chmod 644 $csv_files
    echo "chown database:ccmbase $csv_files"
    chown database:ccmbase $csv_files 2>&1
    echo "/bin/cp -fp $csv_files $CSVDESTDIR"
    /bin/cp -fp $csv_files $CSVDESTDIR 2>&1
#
# Install CSV files into DB
#
    echo "Running DB commands for CSV files."
    for csv_file in $csv_files
    do
      echo "/bin/su -l informix -s /bin/sh -c \"source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -c $CURDIR/$csv_file\""
      /bin/su -l informix -s /bin/sh -c "source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -c $CURDIR/$csv_file" 2>&1
    done
fi

#
# Remove old csv files
#
if [ -n "$old_csv_files" ]; then
    for old_csv_file in $old_csv_files
    do
        echo "/bin/rm -f ${CSVDESTDIR}/$old_csv_file"
        /bin/rm -f ${CSVDESTDIR}/$old_csv_file 2>&1
    done
fi

#
# Copy GIF file and set permissions
#
if [ -n "$gif_files" ] ; then
    echo "chmod 644 $gif_files"
    chmod 644 $gif_files 2>&1
    echo "chown tomcat:tomcat $gif_files"
    chown tomcat:tomcat $gif_files 2>&1
    echo "/bin/cp -fp $gif_files $GIFUSERDESTDIR"
    /bin/cp -fp $gif_files $GIFUSERDESTDIR 2>&1
    echo "/bin/cp -fp $gif_files $GIFADMINDESTDIR"
    /bin/cp -fp $gif_files $GIFADMINDESTDIR 2>&1
    echo "/bin/cp -fp $gif_files $GIFSERVICEDESTDIR"
    /bin/cp -fp $gif_files $GIFSERVICEDESTDIR 2>&1
fi

#
# Copy XML files and change permissions
#
if [ -n "$xml_files" ] ; then
    echo "chmod 644 $xml_files"
    chmod 644 $xml_files 2>&1
    echo "chown database:ccmbase $xml_files"
    chown database:ccmbase $xml_files 2>&1
    echo "/bin/cp -fp $xml_files ${XMLDESTDIR}"
    /bin/cp -fp $xml_files ${XMLDESTDIR} 2>&1

#
# Install XML files into DB
#
    echo "Running DB commands for XML files."
    for xml_file in $xml_files
    do
      echo "/bin/su -l informix -s /bin/sh -c \"source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -x $CURDIR/$xml_file\""
      /bin/su -l informix -s /bin/sh -c "source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -x $CURDIR/$xml_file" 2>&1
    done
fi

#
# Remove old firmware files
#
if [ -n "$old_firmware_files" ]; then
    for old_firmware_file in $old_firmware_files
    do
        echo "/bin/rm -f ${FIRMWAREDESTDIR}/$old_firmware_file"
        /bin/rm -f ${FIRMWAREDESTDIR}/$old_firmware_file 2>&1
    done
fi

#
# Copy firmware files and change permissions
#
if [ -n "$firmware_files" ]; then
    echo "/bin/chmod 770 $firmware_files"
    /bin/chmod 770 $firmware_files 2>&1
    echo "/bin/chown ctftp:ccmbase $firmware_files"
    /bin/chown ctftp:ccmbase $firmware_files 2>&1
    echo "/bin/cp -fp $firmware_files ${FIRMWAREDESTDIR}"
    /bin/cp -fp $firmware_files ${FIRMWAREDESTDIR} 2>&1
fi

#
# Update the device defaults
#
if [ -n "$load_files" ] ; then
    echo "/bin/chmod 770 ${load_files}"
    /bin/chmod 770 ${load_files} 2>&1
    echo "/bin/chown database:ccmbase ${load_files}"
    /bin/chown database:ccmbase ${load_files} 2>&1
    echo "/bin/cp -fp ${load_files} ${LOADFILEDESTDIR}"
    /bin/cp -fp ${load_files} ${LOADFILEDESTDIR} 2>&1
    for load_file in $load_files
    do
      echo "/bin/su -l informix -s /bin/sh -c \"source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -l $CURDIR/$load_file\""
      /bin/su -l informix -s /bin/sh -c "source /usr/local/cm/db/dblenv.bash /usr/local/cm ; source /usr/local/cm/db/informix/local/ids.env ${L2_OPT}; $INSTALLDB -l $CURDIR/$load_file" 2>&1
    done
fi

#
# Copy Help file and set permissions
# (Used if you configure in your CSV that you have help file, and specify filename)
#

exit 0
